#include "config.h"

#include <stdio.h>
#include <unistd.h>
#include <alloca.h>
#include <stdarg.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "util.h"
#include "panic.h"

#ifndef HOST_NAME_MAX
#define HOST_NAME_MAX 256
#endif

int
daemon_init(void)
{
	pid_t pid;
	if ((pid = fork()) < 0)
		return -1;
	else if (pid != 0)
		exit(0);
	setsid();
	chdir("/");
	umask(0);
	return 0;
}


#ifndef HAVE_STRDUP
char *
strdup(const char *s) 
{
        char *ns;
        ns = malloc(strlen(s)+1);
        if(ns)
                strcpy(ns,s);
        return ns;
}
#endif

#ifndef HAVE_ASPRINTF
int
asprintf(char **s, char *fmt, ...)
{
        int cs;
        char *s;
        va_list ap;
        va_start(ap, fmt);
        cs = vsnprintf(NULL, 0, fmt, ap);
        if(cs == -1)
                return -1;
        *s = malloc(cs + 1);
        if(!*s)
                return -1;
        cs = vsnprintf(*s, cs + 1, fmt, ap);
        va_end(ap);
        return cs;
        
}
#endif

int
create_temporary_file(char *dir, char **name)
{

        do {
                int fd;
                create_temporary_filename(dir, name);
                fd = open(*name, O_WRONLY | O_CREAT | O_EXCL, 0666);
                if(fd >= 0 || errno != EEXIST)
                        return fd;
                free(*name);
        } while(1);

}

void
create_temporary_filename(char *dir, char **name)
{
        int ret;
        static char *hostname = NULL;
        static int generation = 0;
        pid_t pid = getpid();
        if(!hostname) {
                char hn[HOST_NAME_MAX];
                ret = gethostname(hn, HOST_NAME_MAX);
                if(ret == -1)
                        panicno("gethostname");
                hostname = strdup(hn);
        }
        bfree((void *)name);
        ret = asprintf(name, "%s/__%s-%u-%u.tmp", dir, hostname, (unsigned)generation++, (unsigned)pid);
        if(ret == -1) {
                panicno("asprintf");
        }
}


void
bfree(void **buf)
{
        if(buf) {
                free(*buf);
                *buf = NULL;
        }
}

void
balloc(void **buf, size_t size)
{
        void *nbuf;
        nbuf = realloc(*buf, size);
        if(!nbuf)
                panicno("realloc");
        *buf = nbuf;
}

int
basprintf(char **s, char *fmt, ...)
{
        int cs;
        va_list ap;
        va_start(ap, fmt);
        cs = vsnprintf(NULL, 0, fmt, ap);
        if(cs == -1)
                return -1;
        balloc((void *)s, cs + 1);
        if(!*s)
                return -1;
        cs = vsnprintf(*s, cs + 1, fmt, ap);
        va_end(ap);
        return cs;
        
}
