#ifndef VSDB_H
#define VSDB_H

#include <inttypes.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdbool.h>

typedef struct vsdb vsdb_t;
typedef struct vsdb_transaction  vsdb_transaction_t;

struct vsdb_info {
        time_t mtime;
        uintmax_t num_entries;
        uint32_t generation;
        char *name;
        uintmax_t key_size;
        uintmax_t data_size;
};

/* opening and closing database */
vsdb_t *vsdb_open(char *name, int mode);
void vsdb_close(vsdb_t *v);

/* sometimes useful to reference count database handles */
void vsdb_ref(vsdb_t *v);
void vsdb_unref(vsdb_t *v);


/* querying database info */
void vsdb_get_info(vsdb_t *vsdb, struct vsdb_info *info);
uintmax_t vsdb_transaction_num_entries(vsdb_transaction_t *vt);



/* transaction management */

/* every call to this MUST be matched by an abort or commit transaction request */
vsdb_transaction_t *vsdb_transaction_begin(vsdb_t *vsdb, int mode); 

/* aborts transaction */
void vsdb_transaction_rollback(vsdb_transaction_t *vt); //TODO  finish

/* returns 0 on success  < 0 on error, and > 0 when transaction should be retried */
int vsdb_transaction_commit(vsdb_transaction_t *vt); 


/* NULL or "" sets default table */
void vsdb_set_current_table(vsdb_transaction_t *vt, char *name);

/* manipulating the database */

void vsdb_clear(vsdb_transaction_t *vt, void *key, size_t keysz);
void vsdb_set(vsdb_transaction_t *vt, void *key, size_t keysz, void *data, size_t datasz);
/* returns 0 on success , < 0 on error and > 0 when entry is not found */
int vsdb_lookup(vsdb_transaction_t *vt, void *key, size_t keysz, uintmax_t *which);

/* examining entries */
void vsdb_aread_entry(vsdb_transaction_t *vt, uintmax_t entry, void **key, size_t *keysz, void **data, size_t *datasz);
void vsdb_read_entry(vsdb_transaction_t *vt, uintmax_t entry, void *key, off_t keyoffset, size_t keysz, void *data, off_t dataoffset, size_t datasz);


/* utility routines */
int vsdb_lookup_rr(vsdb_transaction_t *vt, void *key, size_t keysz, void **data, size_t *datasz);
char *vsdb_lookup_ss(vsdb_transaction_t *vt, char *key);
int vsdb_repair_database(char *s);


/* error handling */
void vsdb_set_verbose_mode(bool val);
void vsdb_set_die_on_error(void);
void vsdb_set_errfn(void (*fn)(char *));

#endif
