#include "config.h"


#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <stdlib.h>
#include <stdbool.h>
#include <signal.h>
#include <unistd.h>
#include <time.h>

#include "vsdb.h"
#include "panic.h"


#define DATABASE "bench.vsdb"

#define TEST_LENGTH 1


static bool done = false;

static void 
alarm_sig(int foo) 
{
        done = true;
}




int
main(int argc, char *argv[])
{
        int ret;
        vsdb_t *v;
        vsdb_transaction_t *vt;
        char *s;
        unsigned ui;
        uintmax_t iterations, attempts;
        char buffer[256];
        char buffer2[256];

        verbose_mode = true;

        signal(SIGALRM, alarm_sig);
        srandom(time(NULL));



        v = vsdb_open(DATABASE, O_RDWR | O_CREAT | O_TRUNC  );
        assert(v);

        iterations  = 0;
        attempts  = 0;
        done = false;
        alarm(TEST_LENGTH);
        while(!done)
        {

                do{
                        attempts++;
                        vt = vsdb_transaction_begin(v, O_RDWR);
                        s = vsdb_lookup_ss(vt, "counter");
                        if(s) {
                                sscanf(s, "%u", &ui);
                                free(s);
                        } else ui = 0;
                        snprintf(buffer, 256, "%u", ui + 1);
                        vsdb_set(vt, "counter", 7, buffer, strlen(buffer));;
                        ret = vsdb_transaction_commit(vt);
                } while(ret > 0);
                if(ret < 0)
                        panicno("vsdb_transaction_commit");
                iterations++;
        }
        vsdb_close(v);

        printf("Incrementing Counter: %f op/s\n", (float)(iterations/(double)TEST_LENGTH));
        printf("Average # attempts:   %f\n", (float)(attempts/(float)iterations));

        v = vsdb_open(DATABASE, O_WRONLY);

        iterations  = 0;
        attempts  = 0;
        done = false;
        alarm(TEST_LENGTH);
        while(!done)
        {
                long int foo;
                foo = random();;
                snprintf(buffer, 256, "k%li", foo);
                snprintf(buffer2, 256, "d%li %li %li %li %li %li %li", foo*2, foo*3, foo*4, foo*5, foo*6, foo*7, foo*8);

                do{
                        attempts++;
                        vt = vsdb_transaction_begin(v, O_WRONLY);
                        vsdb_set(vt, buffer, strlen(buffer), buffer2, strlen(buffer2));;
                        ret = vsdb_transaction_commit(vt);
                } while(ret > 0);
                if(ret < 0)
                        panicno("vsdb_transaction_commit");
                iterations++;
        }
        vsdb_close(v);
        printf("adding data:        %f op/s\n", (float)(iterations/(double)TEST_LENGTH));
        printf("Average # attempts: %f\n", (float)(attempts/(float)iterations));

        v = vsdb_open(DATABASE, O_RDONLY);
        iterations  = 0;
        attempts  = 0;
        done = false;
        alarm(TEST_LENGTH);
        while(!done)
        {
                long int foo;
                foo = random();;
                snprintf(buffer, 256, "k%li", foo);

                do{
                        uintmax_t which;
                        attempts++;
                        vt = vsdb_transaction_begin(v, O_RDONLY);
                        ret = vsdb_lookup(vt, buffer, strlen(buffer), &which);
                        assert(ret >= 0);
                        ret = vsdb_transaction_commit(vt);
                } while(ret > 0);
                if(ret < 0)
                        panicno("vsdb_transaction_commit");
                iterations++;
        }
        vsdb_close(v);
        printf("reading data:       %f op/s\n", (float)(iterations/(double)TEST_LENGTH));
        printf("Average # attempts: %f\n", (float)(attempts/(float)iterations));

        v = vsdb_open(DATABASE, O_RDWR );
        iterations  = 0;
        attempts = 0;
        done = false;
        alarm(TEST_LENGTH);
        while(!done)
        {

                do{
                        attempts++;
                        vt = vsdb_transaction_begin(v, O_RDWR);
                        s = vsdb_lookup_ss(vt, "counter");
                        if(s) {
                                sscanf(s, "%u", &ui);
                                free(s);
                        } else ui = 0;
                        snprintf(buffer, 256, "%u", ui + 1);
                        vsdb_set(vt, "counter", 7, buffer, strlen(buffer));;
                        ret = vsdb_transaction_commit(vt);
                } while(ret > 0);
                if(ret < 0)
                        panicno("vsdb_transaction_commit");
                iterations++;
        }
        vsdb_close(v);

        printf("Incrementing Counter: %f op/s\n", (float)(iterations/(float)TEST_LENGTH));
        printf("Average # attempts:   %f\n", (float)(attempts/(float)iterations));

        //sleep(1);

        return 0;
}

