#include "config.h"

#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>

#include "vsdb.h"
#include "panic.h"

#define PROGRAM_NAME "vsdb"

#define SAMPLE_DATABASE "sample.vsdb"
#define NUM_PROCS 32

#define RET_SUCCESS	0
#define RET_FAILURE	1

static void info_database(char *name);
static void dump_database(char *name);

static void
useage(int ret)
{
	FILE *out;

	if(ret == RET_SUCCESS)
		out = stdout;
	else
		out = stderr;

	fprintf(out, PROGRAM_NAME " version " VERSION ", Copyright (C) 2003 John Meacham\n");
	fprintf(out, "useage: " PROGRAM_NAME " [options] <command> [arguments]\n");
	fprintf(out, "        " PROGRAM_NAME " info <database> - "
	       "print out information about database.\n");
	fprintf(out, "        " PROGRAM_NAME " dump <database> - "
	       "dump entire database to stdout.\n");
	fprintf(out, "        " PROGRAM_NAME " clean <database> - "
	       "clean any cruft left over by crashing clients.\n");
	fprintf(out, "        " PROGRAM_NAME " new <database> - create new database.\n"); 
	fprintf(out, "        " PROGRAM_NAME " help - print out help screen.\n"); 
        fprintf(out, "    options:\n");
        fprintf(out, "       -h print help screen\n");
        fprintf(out, "       -v verbose mode\n");

	exit(ret);
}

int
main(int argc, char *argv[]) 
{
        int ch;
	if(argc < 2)
		useage(RET_FAILURE);
	while ((ch = getopt(argc, argv, "hvo:c:")) != -1) {
		switch(ch) {
		case 'v':
			verbose_mode = true;
			break;
		case -1:
			break;
		case 'h':
			useage(RET_SUCCESS);
		default:
			useage(RET_FAILURE);
		}
	}
        if (optind >= argc) {
		useage(RET_FAILURE);
        }
	if(!strcmp(argv[optind], "new")) {
		if(optind + 1 >= argc) {
			printf("need argument to create database.\n");
			useage(RET_FAILURE);
		}
//		generate_new_key();

	} else if(!strcmp(argv[optind], "info")) {
		if(optind + 1 >= argc) {
			printf("database must be specified.\n");
			useage(RET_FAILURE);
		}
                info_database(argv[optind + 1]);
	} else if(!strcmp(argv[optind], "help")) {
		useage(RET_SUCCESS);
	} else if(!strcmp(argv[optind], "clean")) {
		if(optind + 1 >= argc) {
			printf("database must be specified.\n");
			useage(RET_FAILURE);
		}
		//nym_sign(argv[optind + 1], argv[optind + 2]);
	} else if(!strcmp(argv[1], "dump")) {
		if(optind + 1 >= argc) {
			printf("database must be specified.\n");
			useage(RET_FAILURE);
		}
                dump_database(argv[optind + 1]);
//		nym_verify(argv[optind + 1], argv[optind + 2]);
	} else 
		useage(RET_FAILURE);
	return 0;
}

static void
dump_database(char *name)
{
        uintmax_t i;
        vsdb_t *v;
        vsdb_transaction_t *vt;
        uintmax_t num_entries;
        v = vsdb_open(name, O_RDONLY);
        if(!v) {
                perror(name);
                fprintf(stderr, "Could not open database!\n");
                exit(1);
        }
        vt = vsdb_transaction_begin(v, O_RDONLY);
        if(!vt) {
                fprintf(stderr, "Could not open transaction!\n");
                exit(1);
        }
        num_entries = vsdb_transaction_num_entries(vt);
        //printf("%lu\n", (unsigned long) num_entries);
        for(i = 0; i < num_entries; i++) {
                void *key, *data;
                size_t keysz, datasz;
                vsdb_aread_entry(vt, i, &key, &keysz, &data, &datasz);
                printf("+%lu,%lu:", (unsigned long)keysz, (unsigned long)datasz);
                fwrite(key, keysz, 1, stdout);
                fputs("->", stdout);
                fwrite(data, datasz, 1, stdout);
                putchar('\n');
                free(key);
                free(data);
        }
        printf("\n");
        

        vsdb_transaction_commit(vt);

        
        vsdb_close(v);
}

static void
info_database(char *name)
{
        vsdb_t *v;
        struct vsdb_info vi;        
        v = vsdb_open(name, O_RDONLY);
        if(!v) {
                perror(name);
                fprintf(stderr, "Could not open database!\n");
                exit(1);
        }
        vsdb_get_info(v, &vi);
        printf("Name:       %s\n", vi.name);
        printf("Time:       %lu\n", (unsigned long)vi.mtime);
        printf("Entries:    %lu\n", (unsigned long)vi.num_entries);
        printf("Generation: %lu\n", (unsigned long)vi.generation);
        printf("Key Bytes:  %lu\n", (unsigned long)vi.key_size);
        printf("Data Bytes:  %lu\n", (unsigned long)vi.data_size);
        vsdb_close(v);
}

/*

int
main(int argc, char *argv[])
{
        int ret;
        vsdb_t *v;
        vsdb_transaction_t *vt;
        char *s;
        unsigned ui;
        char buffer[256];
        pid_t arr[NUM_PROCS];
        struct vsdb_info vi;        

        memset(arr, 0, NUM_PROCS*sizeof(pid_t));


        v = vsdb_open(SAMPLE_DATABASE, O_RDONLY);
        vsdb_get_info(v, &vi);
        printf("Name:       %s\n", vi.name);
        printf("Time:       %lu\n", (unsigned long)vi.mtime);
        printf("Entries:    %lu\n", (unsigned long)vi.num_entries);
        printf("Generation: %lu\n", (unsigned long)vi.generation);
        printf("Key Bytes:  %lu\n", (unsigned long)vi.key_size);
        printf("Data Bytes:  %lu\n", (unsigned long)vi.data_size);
        assert(v);
        //printf("num: %lu\n", (unsigned long)v->num);
        //printf("generation: %lu\n", (unsigned long)v->generation);
        vt = vsdb_transaction_begin(v, O_RDONLY);
        s = vsdb_lookup_ss(vt, "rachel");
        printf("rachel->%s\n", s);
        s = vsdb_lookup_ss(vt, "john");
        printf("john->%s\n", s);
        s = vsdb_lookup_ss(vt, "loban");
        printf("loban->%s\n", s);
        vsdb_commit_transaction(vt);

        //fork() ; fork() ; fork() ; fork() ; fork() ; fork() ; 
        for (;;) {                                                                                                                                
                sleep(1);
                int const x = random() % NUM_PROCS;                                                                                                    
                if(arr[x] != 0)
                        kill(arr[x], SIGINT);                                                                                                                         
                arr[x] = fork();                                                                                                                      
                if (0 == arr[x]) while (1) {                                                                                                                    
                        int count = 0;
                        do{
                                count++;
                                fflush(stdout);
                                vt = vsdb_transaction_begin(v, O_RDWR);
                                s = vsdb_lookup_ss(vt, "counter");
                                if(s) {
                                        sscanf(s, "%u", &ui);
                                } else ui = 0;
                                snprintf(buffer, 256, "%u", ui + 1);
                                vsdb_set(vt, "counter", 7, buffer, strlen(buffer));;
                                ret = vsdb_commit_transaction(vt);
                        } while(ret > 0);
                        if(ret < 0)
                                panicno("vsdb_commit_transaction");
                        printf("counter: %.4i %u\n", count, ui);
                }                                                                                                                                     
        }                         

        do{
                printf(".");
                fflush(stdout);
                vt = vsdb_transaction_begin(v, O_RDWR);
                s = vsdb_lookup_ss(vt, "counter");
                if(s) {
                        sscanf(s, "%u", &ui);
                } else ui = 0;
                snprintf(buffer, 256, "%u", ui + 1);
                vsdb_set(vt, "counter", 7, buffer, strlen(buffer));;
                ret = vsdb_commit_transaction(vt);
        } while(ret > 0);
        if(ret < 0)
                panicno("vsdb_commit_transaction");

        printf("counter: %u\n", ui);

        //sleep(1);

        return 0;
}

*/
